Package: bakR
Title: Analyze and Compare Nucleotide Recoding RNA Sequencing Datasets
Version: 1.0.1
Authors@R: 
    person(given = "Isaac",
           family = "Vock",
           role = c("aut", "cre"),
           email = "isaac.vock@gmail.com",
           comment = c(ORCID = "0000-0002-7178-6886"))
Description: Several implementations of a novel Bayesian hierarchical statistical model of nucleotide
	recoding RNA-seq experiments (NR-seq; TimeLapse-seq, SLAM-seq, TUC-seq, etc.)
	for analyzing and comparing NR-seq datasets (see 'Vock and Simon' (2023) <doi:10.1261/rna.079451.122>).
	NR-seq is a powerful extension of RNA-seq that provides information about the kinetics
	of RNA metabolism (e.g., RNA degradation rate constants), which is notably lacking
	in standard RNA-seq data. The statistical model makes maximal use of these high-throughput
	datasets by sharing information across transcripts to significantly improve
	uncertainty quantification and increase statistical power. 'bakR' includes a maximally
	efficient implementation of this model for conservative initial investigations of datasets. 'bakR'
	also provides more highly powered implementations using the probabilistic programming language
	'Stan' to sample from the full posterior distribution. 'bakR' performs multiple-test
	adjusted statistical inference with the output of these model implementations to
	help biologists separate signal from background. Methods to automatically visualize key
	results and detect batch effects are also provided.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.3
Biarch: true
Depends: R (>= 3.5.0)
Imports: purrr, methods, Rcpp (>= 0.12.0), RcppParallel (>= 5.0.1),
        rstan (>= 2.26.0), rstantools (>= 2.1.1), dplyr, tidyr, stats,
        magrittr, Hmisc, ggplot2, data.table
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.26.0), StanHeaders (>=
        2.26.0)
SystemRequirements: GNU make C++17
Suggests: rmarkdown, knitr, DESeq2, pheatmap, Ckmeans.1d.dp, corrplot
VignetteBuilder: knitr
URL: https://simonlabcode.github.io/bakR/
BugReports: https://github.com/simonlabcode/bakR/issues/
NeedsCompilation: yes
Packaged: 2024-01-13 16:37:02 UTC; isaac
Author: Isaac Vock [aut, cre] (<https://orcid.org/0000-0002-7178-6886>)
Maintainer: Isaac Vock <isaac.vock@gmail.com>
Repository: CRAN
Date/Publication: 2024-01-13 17:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 11:01:56 UTC; unix
Archs: bakR.so.dSYM
