# Generated by roxygen2: do not edit by hand

export(CorrectDropout)
export(DissectMechanism)
export(FnPCA)
export(FnPCA2)
export(GSprocessing)
export(Heatmap_kdeg)
export(NSSHeat)
export(QC_checks)
export(QuantifyDropout)
export(Simulate_bakRData)
export(Simulate_relative_bakRData)
export(VisualizeDropout)
export(bakRData)
export(bakRFit)
export(bakRFnData)
export(cBprocess)
export(fast_analysis)
export(fn_process)
export(plotMA)
export(plotVolcano)
export(reliableFeatures)
import(Rcpp)
import(data.table)
import(methods)
importFrom(magrittr,"%>%")
importFrom(rstan,sampling)
useDynLib(bakR, .registration = TRUE)
